From 66cc6b85e13fc49a5b8115c4606d36660c486a97 Mon Sep 17 00:00:00 2001
From: Naofumi Yasufuku <naofumi@yasufuku.dev>
Date: Thu, 3 Jun 2021 22:02:28 +0900
Subject: [PATCH] Fix describe-bindings stuck caused by heavy shadow_lookup()
 loop

* src/keymap.c (describe_vector): If 'shadowed_by' key definition
is nil, do not attempt to find a first key character, which is
shadowed by the same command, in consecutive character table range by
using shadow_lookup() loop.  This loop can cause millions of
Flookup_key() calls: e.g. 'self-insert-command' bindings.
(Bug#48812 Bug#9293)
---
 src/keymap.c | 2 +-
 1 file changed, 1 insertion(+), 1 deletion(-)

diff --git a/src/keymap.c b/src/keymap.c
index fb8eceaec1..a20b877565 100644
--- a/src/keymap.c
+++ b/src/keymap.c
@@ -3062,7 +3062,7 @@ describe_vector (Lisp_Object vector, Lisp_Object prefix, Lisp_Object args,
 
       /* Make sure found consecutive keys are either not shadowed or,
 	 if they are, that they are shadowed by the same command.  */
-      if (CHAR_TABLE_P (vector) && i != starting_i)
+      if (!NILP (shadowed_by) && CHAR_TABLE_P (vector) && i != starting_i)
 	{
 	  Lisp_Object key = make_nil_vector (1);
 	  for (int j = starting_i + 1; j <= i; j++)
-- 
2.31.1

